---------------------------------------------------------------------------
-- SpaceXplorer
-- main.lua
-- Main program
-- credit:
--   BG Music: Cavern and Blade (Open Game Art)
---------------------------------------------------------------------------

-- coordinat
local centerX
local centerY
local shipAnchorY

-- music and font
local bgmusic
local font

-- references for game objects
local ship
local bgspace 
local canon
local stars
local asteroids
local aidkit
local canister
local explosion
local powerup

-- score and button
local score
local hiscore
local gameover
local gameoverImg
local playImg

-- circular collision detection
local function checkCollision()
  for i = #asteroids.rocks, 1, -1 do
    for j = #canon.missile, 1, -1 do
      -- missiles and asteroids
      local dx = canon.missile[j].x - asteroids.rocks[i].x
      local dy = canon.missile[j].y - asteroids.rocks[i].y
      local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2)) -- distance, using hypot
      
      if d <= 16 then
        explosion.emit(asteroids.rocks[i].x, asteroids.rocks[i].y)
        canon.missile[j].life = false
        asteroids.rocks[i].life = false
        score = score + 10
        
        if score > hiscore then 
          hiscore = score 
        end
      end
    end
    
    -- asteroids and ship
    local dx = asteroids.rocks[i].x - ship.x
    local dy = asteroids.rocks[i].y - ship.y
    local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2)) -- distance, using hypot
      
    if d <= 48 then
      love.system.vibrate(1)
      explosion.emit(asteroids.rocks[i].x, asteroids.rocks[i].y)
      asteroids.rocks[i].life = false
      ship.shake()
      ship.shield = ship.shield - 1
      
      if ship.shield == 0 then
        gameover = true
        ship.life = false
        ship.moveTo(ship.x, height + 64)
      end
    end
  end
end

-- show score, hi-schore, and shield-bar
local function showHUD()
  love.graphics.setColor(1, 1, 1, 1)
  if score == hiscore then love.graphics.setColor(1, 1, 0, 1) end
  love.graphics.print('score: ' .. score, 10, 10)
  love.graphics.print('hi-score: ' .. hiscore, width - 125, 10)
  
  if gameover then
    love.graphics.setColor(1, 1, 1, 1)
    love.graphics.draw(gameoverImg, centerX, centerY, 0, 1, 1, 100, 23)
    love.graphics.draw(playImg, centerX, shipAnchorY, 0, 1, 1, 24, 24)
  end
end

function resetGame()
  gameover = false
  score = 0
  aidkit.reset()
  ship.reset()
   
  ship.moveTo(centerX, shipAnchorY)
end

function love.load()  
  width = love.graphics.getHeight()
  height = love.graphics.getWidth()
  
  centerX = width / 2
  centerY = height / 2
  
  shipAnchorY = centerY + 75
  
  bgmusic = love.audio.newSource('cavern.ogg', 'stream')
  bgmusic:setLooping(true)
  
  bgspace = require('bgspace')
  
  ship = require('ship')
  ship.moveTo(centerX, shipAnchorY)
  
  canon = require('canon')
  stars = require('stars')
  asteroids = require('asteroids')
  aidkit = require('aidkit')
  canister = require('canister')
  explosion = require('explosion')
  powerup = require('powerup')
  
  font = love.graphics.newFont('whitrabt.ttf')
  love.graphics.setFont(font)
  
  gameoverImg = love.graphics.newImage('gameover.png')
  playImg = love.graphics.newImage('play.png')
  
  score = 0
  hiscore = 0
  gameover = false
  
  bgmusic:play()
end

function love.update(dt)
  bgspace.update()
  stars.update()
  
  aidkit.update()
  canister.update()
  
  ship.update(dt)
  canon.update()
  
  asteroids.update()
  explosion.update(dt)
  powerup.update(dt)
  
  if ship.fuel <= 0 then
    gameover = true
    ship.moveTo(ship.x, shipAnchorY + 100)
  end
  
  if not gameover then
    asteroids.create(dt)  
    checkCollision()
    
    if aidkit.heal(ship) or canister.fill(ship) then
      powerup.emit(ship.x, ship.y)
    end
  end
end

function love.draw()  
  -- translate and rotate
  love.graphics.translate(0, love.graphics.getHeight())
  love.graphics.rotate(-0.5 * math.pi)
  
  love.graphics.setColor(1, 1, 1, 1)
  bgspace.draw()
  
  stars.draw()
  asteroids.draw()
  canon.draw()
  ship.draw()
  aidkit.draw()
  canister.draw()
  explosion.draw()
  powerup.draw()
  
  showHUD()
end

function love.mousepressed(mx, my, button)
  if button == 1 then
    local x = love.graphics.getHeight() - my
    local y = mx
    if gameover then
      local dx = centerX - x
      local dy = shipAnchorY - y
      local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
      
      if d <= 24 then
        resetGame()
      end
    else
      ship.moveTo(x, shipAnchorY)
      canon.fire(ship.x, shipAnchorY)
    end
  end
end

function love.mousemoved(mx, my)
  if love.mouse.isDown(1) then
    local x = love.graphics.getHeight() - my
    if not gameover then
      ship.moveTo(x, shipAnchorY)
    end
  end
end

function love.keypressed(key)
  if key == 'escape' then
    os.exit()
  end
end
